##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
""" Customizable objects that come from the filesystem (base class).

$Id: FSObject.py,v 1.6 2003/11/03 08:17:41 gotcha Exp $
"""

from string import split
from os import path, stat

import Acquisition, Globals
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import Item
from DateTime import DateTime

from utils import expandpath, getToolByName
from Permissions import View
from Permissions import ViewManagementScreens
from Permissions import ManagePortal

from OFS.Cache import Cacheable

class FSObject(Acquisition.Implicit, Item, Cacheable):
    """FSObject is a base class for all filesystem based look-alikes.
    
    Subclasses of this class mimic ZODB based objects like Image and
    DTMLMethod, but are not directly modifiable from the management
    interface. They provide means to create a TTW editable copy, however.
    """

    # Always empty for FS based, non-editable objects.
    title = ''

    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    _file_mod_time = 0
    _parsed = 0

    def __init__(self, id, filepath, fullname=None, properties=None):
        if properties:
            # Since props come from the filesystem, this should be
            # safe.
            self.__dict__.update(properties)
            if fullname and properties.get('keep_extension', 0):
                id = fullname

            cache = properties.get('cache')
            if cache:
                self.ZCacheable_setManagerId(cache)

        self.id = id
        self.__name__ = id # __name__ is used in traceback reporting
        self._filepath = filepath
        fp = expandpath(self._filepath)
        
        try: self._file_mod_time = stat(fp)[8]
        except: pass
        self._readFile(0)

    security.declareProtected(ViewManagementScreens, 'manage_doCustomize')
    def manage_doCustomize(self, folder_path, RESPONSE=None, root=None):
        """Makes a ZODB Based clone with the same data.

        Calls _createZODBClone for the actual work.
        """

        obj = self._createZODBClone()
        
        id = obj.getId()
        if root is None:
            rootFolder = getToolByName(self,'portal_skins')
        else:
            rootFolder = root
        if folder_path == '.':
            folder = rootFolder.restrictedTraverse(())
        else:
            fpath = tuple(split(folder_path, '/'))
            folder = rootFolder.restrictedTraverse(fpath)
        folder._verifyObjectPaste(obj, validate_src=0)
        folder._setObject(id, obj)

        if RESPONSE is not None:
            RESPONSE.redirect('%s/%s/manage_main' % (
                folder.absolute_url(), id))

    def _createZODBClone(self):
        """Create a ZODB (editable) equivalent of this object."""
        raise NotImplementedError, "This should be implemented in a subclass."

    def _readFile(self, reparse):
        """Read the data from the filesystem.
        
        Read the file indicated by exandpath(self._filepath), and parse the
        data if necessary.  'reparse' is set when reading the second
        time and beyond.
        """
        raise NotImplementedError, "This should be implemented in a subclass."

    # Refresh our contents from the filesystem if that is newer and we are
    # running in debug mode.
    def _updateFromFS(self):
        parsed = self._parsed
        if not parsed or Globals.DevelopmentMode:
            fp = expandpath(self._filepath)
            try:    mtime=stat(fp)[8]
            except: mtime=0
            if not parsed or mtime != self._file_mod_time:
                # if we have to read the file again, remove the cache
                self.ZCacheable_invalidate()
                self._readFile(1)
                self._file_mod_time = mtime
                self._parsed = 1

    security.declareProtected(View, 'get_size')
    def get_size(self):
        """Get the size of the underlying file."""
        fp = expandpath(self._filepath)
        return path.getsize(fp)

    security.declareProtected(View, 'getModTime')
    def getModTime(self):
        """Return the last_modified date of the file we represent.

        Returns a DateTime instance.
        """
        self._updateFromFS()
        return DateTime(self._file_mod_time)

    security.declareProtected(ViewManagementScreens, 'getObjectFSPath')
    def getObjectFSPath(self):
        """Return the path of the file we represent"""
        self._updateFromFS()
        return self._filepath

Globals.InitializeClass(FSObject)

class BadFile( FSObject ):
    """
        Represent a file which was not readable or parseable
        as its intended type.
    """
    meta_type = 'Bad File'
    icon = 'p_/broken'

    BAD_FILE_VIEW = """\
<dtml-var manage_page_header>
<dtml-var manage_tabs>
<h2> Bad Filesystem Object: &dtml-getId; </h2>

<h3> File Contents </h3>
<pre>
<dtml-var getFileContents>
</pre>

<h3> Exception </h3>
<pre>
<dtml-var getExceptionText>
</pre>
<dtml-var manage_page_footer>
"""

    manage_options=(
        {'label':'Error', 'action':'manage_showError'},
        )

    def __init__( self, id, filepath, exc_str=''
                , fullname=None, properties=None):
        id = fullname or id # Use the whole filename.
        self.exc_str = exc_str
        self.file_contents = ''
        FSObject.__init__(self, id, filepath, fullname, properties)

    security = ClassSecurityInfo()

    showError = Globals.HTML( BAD_FILE_VIEW )
    security.declareProtected(ManagePortal, 'manage_showError')
    def manage_showError( self, REQUEST ):
        """
        """
        return self.showError( self, REQUEST )

    security.declarePrivate( '_readFile' )
    def _readFile( self, reparse ):
        """Read the data from the filesystem.
        
        Read the file indicated by exandpath(self._filepath), and parse the
        data if necessary.  'reparse' is set when reading the second
        time and beyond.
        """
        try:
            fp = expandpath(self._filepath)
            file = open(fp, 'rb')
            try:
                data = self.file_contents = file.read()
            finally:
                file.close()
        except:  # No errors of any sort may propagate
            data = self.file_contents = None #give up
        return data
    
    security.declarePublic( 'getFileContents' )
    def getFileContents( self ):
        """
            Return the contents of the file, if we could read it.
        """
        return self.file_contents
    
    security.declarePublic( 'getExceptionText' )
    def getExceptionText( self ):
        """
            Return the exception thrown while reading or parsing
            the file.
        """
        return self.exc_str


Globals.InitializeClass( BadFile )



# Copyright (C) 2004 by Dr. Dieter Maurer, Eichendorffstr. 23, D-66386 St. Ingbert, Germany
# see "doc/TRUSTED_LICENSE.txt" for details
#       $Id: TrustedPythonScriptMixin.py,v 1.1 2004/09/06 17:00:46 faassen Exp $
'''Auxiliary mixin class to implement trusted PythonScripts.'''
import __builtin__
from operator import getitem

from ExtensionClass import Base
from RestrictedPython.RestrictionMutator import \
     RestrictionMutator, _print_target_name
from RestrictedPython.RCompile import RFunction, compileAndTuplize

from Products.PythonScripts.PythonScript import PythonScript

from ReuseUtils import rebindFunction


class TrustedPythonScriptMixin(Base):
  _newfun = rebindFunction(PythonScript._newfun.im_func,
                           safe_builtins=__builtin__.__dict__,
                           _getattr_=getattr,
                           _getitem_=getitem,
                           _write_=lambda ob:ob,
                           )

  def _compiler(self, *args, **kw):
    kw['globals'] = kw['globalize']
    del kw['globalize']
    gen = RFunction(*args, **kw)
    gen.rm = RestrictionMutator()
    return compileAndTuplize(gen)


class RestrictionMutator(RestrictionMutator):
  '''a 'RestrictionMutator' without restrictions.'''
  def _ok(self, *args, **kw): return 1
  checkName = checkAttrName = _ok

  def _default(self, node, walker): return walker.defaultVisitNode(node)

  visitGetattr = visitSubscript = visitAssAttr = visitExec = visitClass \
                 = visitModule = visitAugAssign = visitImport \
                 = _default

  def visitPrint(self, node, walker):
    """we add the current print target if no target is specified."""
    node = walker.defaultVisitNode(node)
    self.funcinfo._print_used = 1
    if node.dest is None: node.dest = _print_target_name
    return node

  visitPrintnl = visitPrint 



# Copyright (C) 2004 by Dr. Dieter Maurer, Eichendorffstr. 23, D-66386 St. Ingbert, Germany
# see "doc/TRUSTED_LICENSE.txt" for details
#       $Id: TrustedFSPythonScript.py,v 1.2 2005/07/27 10:20:00 walco Exp $
'''FSPythonScript unrestricted by Zopes security.'''

from FSPythonScript import FSPythonScript, registerFileExtension, Cacheable

from ReuseUtils import rebindFunction
from ExtensionClassUtils import _UnCustomizable
from TrustedPythonScriptMixin import TrustedPythonScriptMixin
from TrustedPythonScript import TrustedPythonScript


class TrustedFSPythonScript(_UnCustomizable, TrustedPythonScriptMixin,
                            FSPythonScript):
  meta_type = 'Trusted Filesystem Python Script'

  _write = rebindFunction(FSPythonScript._write.im_func,
                          PythonScript=TrustedPythonScript,
                          )

registerFileExtension('xpy', TrustedFSPythonScript)



# Copyright (C) 2004 by Dr. Dieter Maurer, Eichendorffstr. 23, D-66386 St. Ingbert, Germany
# see "LICENSE.txt" for details
#       $Id: ExtensionClassUtils.py,v 1.1 2005/07/27 10:20:00 walco Exp $
'''Utilities'''

from ExtensionClass import Base

class _UnCustomizable(Base):
  '''mixin class to prevent customization.'''
  def manage_doCustomize(self):
    "do not allow customization"
    raise TypeError('This object does not support customization')



ManageProperties = 'Manage properties'
ModifyPortalContent = 'Modify portal content'
View = 'View'

FTIDATA_ACTIONS = (
      { 'id' : 'Action Tests'
      , 'meta_type' : 'Dummy'
      , 'actions' : (
            { 'id':'view',
              'name':'View',
              'action':'string:',
              'permissions':('View',),
              'category':'object',
              'visible':1 }
          , { 'name':'Edit',                    # Note: No ID passed
              'action':'string:${object_url}/foo_edit',
              'permissions':('Modify',),
              'category':'object',
              'visible':1 }
          , { 'name':'Object Properties',       # Note: No ID passed
              'action':'string:foo_properties',
              'permissions':('Modify',),
              'category':'object',
              'visible':1 }
          , { 'id':'slot',
              'action':'string:foo_slot',
              'category':'object',
              'visible':0 }
          )
      }
    ,
    )

FTIDATA_DUMMY = (
      { 'id' : 'Dummy Content'
      , 'title' : 'Dummy Content Title'
      , 'meta_type' : 'Dummy'
      , 'product' : 'FooProduct'
      , 'factory' : 'addFoo'
      , 'actions' : (
            { 'name':'View',
              'action':'string:view',
              'permissions':('View',) }
          , { 'name':'View2',
              'action':'string:view2',
              'permissions':('View',) }
          , { 'name':'Edit',
              'action':'string:edit',
              'permissions':('forbidden permission',) }
          )
      }
    ,
    )

FTIDATA_CMF13 = (
      { 'id' : 'Dummy Content 13'
      , 'meta_type' : 'Dummy'
      , 'description' : (
           'Dummy Content.')
      , 'icon' : 'dummy_icon.gif'
      , 'product' : 'FooProduct'
      , 'factory' : 'addFoo'
      , 'immediate_view' : 'metadata_edit_form'
      , 'actions' : (
            { 'id':'view',
              'name':'View',
              'action':'dummy_view',
              'permissions':(View,) }
          , { 'id':'edit',
              'name':'Edit',
              'action':'dummy_edit_form',
              'permissions':(ModifyPortalContent,) }
          , { 'id':'metadata',
              'name':'Metadata',
              'action':'metadata_edit_form',
              'permissions':(ModifyPortalContent,) }
          )
      }
    ,
    )

FTIDATA_CMF13_FOLDER = (
      { 'id' : 'Dummy Folder 13'
      , 'meta_type' : 'Dummy Folder'
      , 'description' : (
           'Dummy Folder.')
      , 'icon' : 'dummy_icon.gif'
      , 'product' : 'FooProduct'
      , 'factory' : 'addFoo'
      , 'filter_content_types' : 0
      , 'immediate_view' : 'dummy_edit_form'
      , 'actions' : (
            { 'id':'view',
              'name':'View',
              'action':'',
              'permissions':(View,),
              'category':'folder' }
          , { 'id':'edit',
              'name':'Edit',
              'action':'dummy_edit_form',
              'permissions':(ManageProperties,),
              'category':'folder' }
          , { 'id':'localroles',
              'name':'Local Roles',
              'action':'folder_localrole_form',
              'permissions':(ManageProperties,),
              'category':'folder' }
          )
      }
    ,
    )


STI_SCRIPT = """\
## Script (Python) "addBaz"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=container, id
##title=
##
product = container.manage_addProduct['FooProduct']
product.addFoo(id)
item = getattr(container, id)
return item
"""



import Zope
from unittest import main
from Products.FileSystemSite.tests.base.utils import build_test_suite

def test_suite():

    return build_test_suite('Products.FileSystemSite.tests',[
        'test_DirectoryView',
        'test_FSFile',
        'test_FSPythonScript',
        'test_FSPageTemplate',
        'test_FSImage',
        'test_FSSecurity',
        ])

if __name__ == '__main__':
    main(defaultTest='test_suite')



# Copyright (C) 2004 by Dr. Dieter Maurer, Eichendorffstr. 23, D-66386 St. Ingbert, Germany
# see "doc/TRUSTED_LICENSE.txt" for details
#       $Id: TrustedPythonScript.py,v 1.1 2004/09/06 17:00:46 faassen Exp $
'''Python Script unrestricted by Zopes security.

CAUTION: Almost surely, you do not want to make this available.
'''

from Products.PythonScripts.PythonScript import PythonScript

from TrustedPythonScriptMixin import TrustedPythonScriptMixin

class TrustedPythonScript(TrustedPythonScriptMixin, PythonScript):
  '''Python Script unrestriced by Zopes security.'''
  meta_type = 'Trusted Python Script'




import DirectoryView

import FSImage
import FSFile
import FSPropertiesObject
import FSSTXMethod
import FSDTMLMethod
import FSPageTemplate
import FSZSQLMethod
import FSPythonScript
import FSExternalMethod
import TrustedFSPythonScript

def initialize(context):

    context.registerClass(
            DirectoryView.DirectoryViewSurrogate,
            constructors=(('manage_addDirectoryViewForm',
                           DirectoryView.manage_addDirectoryViewForm),
                          DirectoryView.manage_addDirectoryView,
                          DirectoryView.manage_listAvailableDirectories,
                          ),
            icon='images/dirview.gif'            
    )

    utils.registerIcon(FSPageTemplate.FSPageTemplate,
                       'images/fspt.gif', globals())
    utils.registerIcon(FSDTMLMethod.FSDTMLMethod,
                       'images/fsdtml.gif', globals())
    utils.registerIcon(FSPythonScript.FSPythonScript,
                       'images/fspy.gif', globals())
    utils.registerIcon(TrustedFSPythonScript.TrustedFSPythonScript,
                       'images/fspy.gif', globals())
    utils.registerIcon(FSImage.FSImage,
                       'images/fsimage.gif', globals())
    utils.registerIcon(FSFile.FSFile,
                       'images/fsfile.gif', globals())
    utils.registerIcon(FSPropertiesObject.FSPropertiesObject,
                       'images/fsprops.gif', globals())
    utils.registerIcon(FSZSQLMethod.FSZSQLMethod,
                       'images/fssqlmethod.gif', globals())



# Copyright (C) 2004 by Dr. Dieter Maurer, Eichendorffstr. 23, D-66386 St. Ingbert, Germany
# see "LICENSE.txt" for details
#       $Id: ReuseUtils.py,v 1.1 2004/09/06 17:00:46 faassen Exp $

from new import function

def rebindFunction(f,rebindDir=None,**rebinds):
  '''return *f* with some globals rebound.'''
  d= {}
  if rebindDir : d.update(rebindDir)
  if rebinds: d.update(rebinds)
  if not d: return f
  f= getattr(f,'im_func',f)
  fd= f.func_globals.copy()
  fd.update(d)
  nf= function(f.func_code,fd,f.func_name,f.func_defaults or ())
  nf.__doc__= f.__doc__
  if f.__dict__ is not None: nf.__dict__= f.__dict__.copy()
  return nf



from string import strip, split
from os import path, stat

import Globals
import Acquisition

from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.ExternalMethod.ExternalMethod import ExternalMethod

from utils import _dtmldir
from Permissions import ViewManagementScreens, View, FTPAccess
from DirectoryView import registerFileExtension, registerMetaType, expandpath
from FSObject import FSObject

import Permissions

## still need hookpoint for reloading metadata

class FSExternalMethod(FSObject, ExternalMethod):
    
    meta_type = 'FileSystem External Method'

    manage_options=(
        (
            {'label':'Customize', 'action':'manage_main'},
            {'label':'Test',
             'action':'',
             'help':('ExternalMethod','External-Method_Try-It.stx')},   
            )
        )

    security = ClassSecurityInfo()
    security.declareObjectProtected( Permissions.View )

    security.declareProtected( Permissions.ViewManagementScreens
                             , 'manage_main')
    manage_main = Globals.DTMLFile( 'custstx', _dtmldir )

    allowed_params = ('title', 'module', 'function')

    def _createZODBClone(self):
        """
            Create a ZODB (editable) equivalent of this object.
        """
        # XXX:  do this soon
        raise NotImplemented, "not needed yet"

    def _readFile( self, reparse ):

        fp = expandpath( self._filepath )
        file = open( fp, 'r' ) 
        parameters = {}
        
        try:
            for line in file.readlines():
                line = line.strip()
                if not line:
                    continue                    
                if line[0] == '#':
                    continue
                key, value = line.split(':')
                parameters[key.strip()] = value.strip()
                
        finally:
            file.close()

        for k in parameters.keys():
            if k not in self.allowed_params:
                del parameters[k]
        
        self.manage_edit(**parameters)

    if Globals.DevelopmentMode:
        # Provide an opportunity to update the properties.
        def __of__(self, parent):
            try:
                self = Acquisition.ImplicitAcquisitionWrapper(self, parent)
                self._updateFromFS()
                return self
            except:
                from zLOG import LOG, ERROR
                import sys
                LOG('FS Z SQL Method',
                    ERROR,
                    'Error during __of__',
                    error=sys.exc_info())
                raise

Globals.InitializeClass(FSExternalMethod)

registerFileExtension('ext', FSExternalMethod)
registerMetaType('External Method', FSExternalMethod)





## Config this should get dumped to a text config file
## instead of python code

import os
from os import path
from Acquisition import aq_base, aq_inner, aq_parent
from Globals import package_home
from Interface import Base

class IExporter: 
  
  """
  Interfaces are sugar
  """

  def export(self, object, directory): 
    """
    take an object and put a file system representation
    of that object in directory
    """
  
class DTMLExporter:
  
  __implements__ = (IExporter,)
  
  allowed_endings = ('css', 'js', 'html')
  
  def export(self, object, directory): 

    id = object.getId()

    #if not id[id.rfind('.')+1:] in self.allowed_endings:
    id = id + '.dtml'

    source = object.raw
    
    fh = open(path.join(directory,id), 'w')
    fh.write(source)
    fh.close()
    
class ScriptExporter:

  __implements__ = (IExporter,)
  
  allowed_endings = ('ps',)
  
  def export(self, object, directory):
    
    id = object.getId()
    
    #if not id[id.rfind('.')+1:] in self.allowed_endings:
    id = id + '.ps'
    
    source = object.document_src()
    
    fh = open(path.join(directory,id), 'w')
    fh.write(source)
    fh.close()

class ExternalMethodExporter:

  __implements__ = (IExporter,)

  allowed_endings = ('ext',)

  def export (self, object, directory):

    id = object.getId()

    #if not id[id.rfind('.')+1:] in self.allowed_endings:
    id = id + '.ext'

    fh = open(path.join(directory,id),'w')

    print >> fh, '# external method'
    print >> fh, ' title  : %s'%object.title
    print >> fh, ' module : %s'%object._module
    print >> fh, ' function : %s'%object._function

    fh.close()
    
   
class ImageExporter:

  __implements__ = (IExporter,)  
  
  allowed_endings = ('gif','png','jpg','jpeg')
  
  def export(self, object, directory):

    id = object.getId()

    if id[id.rfind('.')+1:] not in self.allowed_endings:
      return
    
    fh = open(path.join(directory,id), 'w')
    
    data = obj.data
    
    if type( data) == type (''):
      fh.write(data)
      
    else:
      while data is not None:
        fh.write(data.data)
        data = data.next

class SQLExporter:

  __implements__ = (IExporter,)  
  
  allowed_endings = ('zsql',)
  
  default_db = 'ardyssdb' ## we need this because of the funky ReadConsistentZSQL

  def export(self, object, directory):
    
    id = object.getId()
    
    #if id[id.rfind('.')+1:] not in self.allowed_endings:
    id = id+'.zsql'
      
    meta_data = {}
    
    meta_data['connection_id'] = getattr(aq_base(object),
                                         'connection_id', 
                                         self.default_db)
    
    meta_data['max_cache']  = getattr(object, 'max_cache_', 100)
    meta_data['max_rows']   = getattr(object, 'max_rows_', 0)
    meta_data['cache_time'] = getattr(object, 'cache_time_', 0)
    meta_data['title']      = getattr(object, 'title', '')
    meta_data['arguments']  = getattr(object, 'arguments_src', '').replace('\n',',')
    
    param_def = """
    <dtml-comment>
    connection_id:%(connection_id)s
    arguments:%(arguments)s
    max_cache:%(max_cache)s
    max_rows:%(max_rows)s
    </dtml-comment>
    """%meta_data

    fh = open(path.join(directory,id), 'w')
    fh.write(param_def+object.src)
    fh.close()
    
    
meta_type_mapping = {'Z SQL Method':SQLExporter(),
                     'ReadConsistentZSQL':SQLExporter(),
                     'Script (Python)':ScriptExporter(),
                     'DTML Method':DTMLExporter(),
                     'DTML Document':DTMLExporter(),
                     'External Method':ExternalMethodExporter()
                     }


#export_directory = '/home/ender/codeit/Products/SkinZ/export'
export_directory = '/home/kapil/export'
#export_directory = '/projects/ardyss/dev/Products/ArdyssSite/export'

def recurse_export(container, directory):

  print  
  print
  print 'exportinging', container.getId(), directory
  
  objs = container.objectValues(meta_type_mapping.keys())
  
  print len(objs)
  
  for o in objs:
    print o.getId(), o.meta_type
    handler = meta_type_mapping.get(o.meta_type)
    handler.export(o, directory)
    
  print    

  containers = container.objectValues('Folder')
  for c in containers:
    d = path.join(directory,  c.getId())
    try:
      os.mkdir(d)
    except OSError:  # hopefully it already exists ;)
      pass
    recurse_export(c, d)
  
def export_container(container, export_directory=export_directory):
  
  #path = package_home(globals())
  directory = export_directory
  
  try:
    os.mkdir(directory)
  except: pass

  recurse_export(container, directory)
            
  return 'fuck ya'

def main():
  
  container = load_container()
  export_container(container)
      
#if __name__ == '__main__':
#  main()
